<?php

namespace Vidgyor\Schedule\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Vidgyor\Schedule\Http\Requests\ScheduleRequest;
use Vidgyor\Schedule\Interfaces\ScheduleRepositoryInterface;
use Vidgyor\Schedule\Models\Schedule;

/**
 * Resource controller class for schedule.
 */
class ScheduleResourceController extends BaseController
{

    /**
     * Initialize schedule resource controller.
     *
     * @param type ScheduleRepositoryInterface $schedule
     *
     * @return null
     */
    public function __construct(ScheduleRepositoryInterface $schedule)
    {
        parent::__construct();
        $this->repository = $schedule;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Vidgyor\Schedule\Repositories\Criteria\ScheduleResourceCriteria::class);
    }

    /**
     * Display a list of schedule.
     *
     * @return Response
     */
    public function index(ScheduleRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Vidgyor\Schedule\Repositories\Presenter\SchedulePresenter::class)
                ->$function();
        }

        $schedules = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('schedule::schedule.names'))
            ->view('schedule::schedule.index', true)
            ->data(compact('schedules', 'view'))
            ->output();
    }

    /**
     * Display schedule.
     *
     * @param Request $request
     * @param Model   $schedule
     *
     * @return Response
     */
    public function show(ScheduleRequest $request, Schedule $schedule)
    {

        if ($schedule->exists) {
            $view = 'schedule::schedule.show';
        } else {
            $view = 'schedule::schedule.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('schedule::schedule.name'))
            ->data(compact('schedule'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new schedule.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ScheduleRequest $request)
    {

        $schedule = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('schedule::schedule.name')) 
            ->view('schedule::schedule.create', true) 
            ->data(compact('schedule'))
            ->output();
    }

    /**
     * Create new schedule.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ScheduleRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $schedule                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('schedule::schedule.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('schedule/schedule/' . $schedule->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/schedule/schedule'))
                ->redirect();
        }

    }

    /**
     * Show schedule for editing.
     *
     * @param Request $request
     * @param Model   $schedule
     *
     * @return Response
     */
    public function edit(ScheduleRequest $request, Schedule $schedule)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('schedule::schedule.name'))
            ->view('schedule::schedule.edit', true)
            ->data(compact('schedule'))
            ->output();
    }

    /**
     * Update the schedule.
     *
     * @param Request $request
     * @param Model   $schedule
     *
     * @return Response
     */
    public function update(ScheduleRequest $request, Schedule $schedule)
    {
        try {
            $attributes = $request->all();

            $schedule->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('schedule::schedule.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('schedule/schedule/' . $schedule->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('schedule/schedule/' . $schedule->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the schedule.
     *
     * @param Model   $schedule
     *
     * @return Response
     */
    public function destroy(ScheduleRequest $request, Schedule $schedule)
    {
        try {

            $schedule->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('schedule::schedule.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('schedule/schedule/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('schedule/schedule/' . $schedule->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple schedule.
     *
     * @param Model   $schedule
     *
     * @return Response
     */
    public function delete(ScheduleRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('schedule::schedule.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('schedule/schedule'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/schedule/schedule'))
                ->redirect();
        }

    }

    /**
     * Restore deleted schedules.
     *
     * @param Model   $schedule
     *
     * @return Response
     */
    public function restore(ScheduleRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('schedule::schedule.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/schedule/schedule'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/schedule/schedule/'))
                ->redirect();
        }

    }

}
